"""
semgrep plugin:

> griffon plugins semgrep setup

"""
import logging
import subprocess

import click

logger = logging.getLogger("griffon")


@click.group(help="semgrep plugin ")
@click.pass_context
def plugins(ctx):
    """semgrep plugin"""
    pass


try:
    # ensure semgrep is installed
    import semgrep  # noqa

    @plugins.command(help="Run this first, installs semgrep")
    @click.argument("scan_dir", required=False, type=click.Path(exists=True))
    @click.pass_context
    def scan(ctx, scan_dir):
        if not scan_dir:
            scan_dir = click.prompt("Please enter a valid directory path to scan")
        subprocess.run(["semgrep", "--config", "auto", scan_dir])

except Exception:
    pass


@plugins.command(help="Run this first, install semgrep python module")
@click.pass_context
def setup(ctx):
    subprocess.run(["pip", "install", "semgrep"])



import inspect
from datetime import datetime
from enum import Enum
from itertools import chain
from types import ModuleType
from typing import Callable, Optional, Union, get_args, get_origin

import click
from osidb_bindings.bindings.python_client.types import OSIDBModel

from griffon import get_config_option


def abort_if_false(ctx, param, value: bool):
    """Helper callback for aborting the command if confirmation check fails"""
    if not value:
        ctx.abort()


def multivalue_params_to_csv(params: dict) -> dict:
    """
    convert multivalue params represented as tuples or lists
    into csv string
    """
    for param_name, param_value in params.items():
        if isinstance(param_value, (tuple, list)):
            params[param_name] = ",".join(params[param_name])
    # TODO - ugly hack to fix osidb bindings ... this should be fixed at some point
    rename_params = {}
    for param_name, param_value in params.items():
        rename_params[param_name.replace("__", "_")] = param_value
    return rename_params


def safe_issubclass(cls: type, class_or_tuple):
    """Safe variant of issubclass which checks for first argument being a class"""
    return inspect.isclass(cls) and issubclass(cls, class_or_tuple)


def to_option_type(type_):
    """Convert type to the Click option type"""

    is_multiple = False
    if get_origin(type_) is list:
        option_type = get_args(type_)[0]
        is_multiple = True
    elif get_origin(type_) == Union and all(safe_issubclass(arg, Enum) for arg in get_args(type_)):
        option_type = click.Choice(
            list(chain.from_iterable([list(enum) for enum in get_args(type_)]))
        )
    elif safe_issubclass(type_, Enum):
        option_type = click.Choice(list(type_))
    elif type_ is datetime:
        option_type = click.DateTime()
    else:
        option_type = type_

    return option_type, is_multiple


def filter_request_fields(fields: dict, exclude: list[str]):
    keep = {}
    for field, field_type in fields.items():
        # Filter out related models
        if safe_issubclass(field_type, OSIDBModel) or (
            get_origin(field_type) is list and safe_issubclass(get_args(field_type)[0], OSIDBModel)
        ):
            continue

        # Filter out excluded fields
        if field in exclude:
            continue

        keep[field] = field_type

    return keep


def get_editor():
    return get_config_option("default", "editor", "vi")


def query_params_options(
    entity: str, endpoint_module: ModuleType, options_overrides: Optional[dict[dict]] = None
) -> Callable:
    """
    Decorator which obtains all query parameters from the given endpoint module and adds
    them as `click.option` with respective type

    Type handling:
        basic types (std, int, bool, etc.) - native
        enums - via `click.Choice`
        lists - multiple option

    For each param option, variable, type, help and multiple can be overriden via
    `options_overrides`
    """
    if options_overrides is None:
        options_overrides = {}

    def inner(fn):
        wrapper = fn
        for query_param, param_type in endpoint_module.QUERY_PARAMS.items():
            option_type, is_multiple = to_option_type(param_type)
            option_params = {
                "option": f"--{query_param.replace('_','-')}",
                "variable": query_param,
                "type": option_type,
                "help": f"{entity.capitalize()} {query_param.replace('_',' ')}",
                "multiple": is_multiple,
            }
            option_override = options_overrides.get(query_param, {})
            option_params.update(
                (override, option_override[override])
                for override in option_params.keys() & option_override.keys()
            )
            wrapper = (
                click.option(
                    option_params.pop("option"), option_params.pop("variable"), **option_params
                )
            )(wrapper)
        return wrapper

    return inner


def request_body_options(
    endpoint_module: ModuleType, exclude: Optional[list[str]] = None
) -> Callable:
    """
    Decorator which obtains all request body fields from the given endpoint module
    them as `click.option` with respective type

    Type handling:
        basic types (std, int, bool, etc.) - native
        enums - via `click.Choice`
        lists - multiple option

    List of the excluded fields may be supplied
    """
    if exclude is None:
        exclude = []

    def inner(fn):
        wrapper = fn

        request_body_type = getattr(endpoint_module, "REQUEST_BODY_TYPE", None)
        if request_body_type is None:
            return wrapper

        fields = filter_request_fields(request_body_type.get_fields(), exclude=exclude)
        for field, field_type in fields.items():
            option_type, is_multiple = to_option_type(field_type)
            option_params = {
                "option": f"--{field.replace('_','-')}",
                "variable": field,
                "type": option_type,
                "help": f"{request_body_type.__name__} {field.replace('_',' ')}",
                "multiple": is_multiple,
            }

            wrapper = (
                click.option(
                    option_params.pop("option"), option_params.pop("variable"), **option_params
                )
            )(wrapper)
        return wrapper

    return inner



"""
corgi entity operations

"""
import concurrent.futures
import logging

import click
import requests
from component_registry_bindings.bindings.python_client.api.v1 import (
    v1_builds_list,
    v1_builds_retrieve,
    v1_channels_list,
    v1_channels_retrieve,
    v1_components_list,
    v1_components_retrieve,
    v1_product_streams_list,
    v1_product_streams_retrieve,
    v1_product_variants_list,
    v1_product_variants_retrieve,
    v1_product_versions_list,
    v1_product_versions_retrieve,
    v1_products_list,
    v1_products_retrieve,
)
from component_registry_bindings.bindings.python_client.models import (
    Channel,
    Component,
    Product,
    ProductStream,
    ProductVariant,
    ProductVersion,
    SoftwareBuild,
)

from griffon import (
    COMMUNITY_COMPONENTS_API_URL,
    CORGI_API_URL,
    CommunityComponentService,
    progress_bar,
)
from griffon.autocomplete import (
    get_component_purls,
    get_product_stream_names,
    get_product_stream_ofuris,
    get_product_version_ofuris,
)
from griffon.commands.entities.helpers import (
    multivalue_params_to_csv,
    query_params_options,
)
from griffon.output import console, cprint

logger = logging.getLogger("griffon")

default_conditions: dict = {}


@click.group(name="community-component-registry")
@click.pass_context
def commmunity_components_grp(ctx):
    pass


# COMPONENTS


@commmunity_components_grp.group(help=f"{COMMUNITY_COMPONENTS_API_URL}/api/v1/components")
@click.pass_context
def components(ctx):
    """Corgi Components."""


@components.command(name="list")
@click.argument("component_name", required=False)
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@query_params_options(
    entity="Component",
    endpoint_module=v1_components_list,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Component))},
    },
)
@click.pass_context
@progress_bar
def list_components(ctx, strict_name_search, component_name, **params):
    # TODO: handle pagination
    # TODO: handle output
    is_params_empty = [False for v in params.values() if v]
    if not component_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    if strict_name_search:
        params["name"] = component_name
    elif component_name:
        params["re_name"] = component_name

    if not params["include_fields"]:
        params[
            "include_fields"
        ] = "link,uuid,purl,nvr,version,type,name,upstreams,related_url,download_url"

    session = CommunityComponentService.create_session()
    params = multivalue_params_to_csv(params)

    logger.debug("starting parallel http requests")
    component_cnt = session.components.retrieve_list(**params).count
    logger.debug(component_cnt)
    if component_cnt < 3000000:
        with concurrent.futures.ThreadPoolExecutor() as executor:
            futures = []
            components = list()
            for batch in range(0, component_cnt, 1200):
                params["offset"] = batch
                params["limit"] = 1200
                futures.append(
                    executor.submit(
                        session.components.retrieve_list,
                        **params,
                    )
                )

            for future in concurrent.futures.as_completed(futures):
                try:
                    components.extend(future.result().results)
                except Exception as exc:
                    logger.warning("%r generated an exception: %s" % (future, exc))

            data = sorted(components, key=lambda d: d.purl)
            return cprint(data, ctx=ctx)
    else:
        console.warning("Too many components.")


@components.command(name="get")
@click.argument("component_id", required=False)
@click.option("--purl", shell_complete=get_component_purls, help="purls must be quoted!")
@query_params_options(
    entity="Component",
    endpoint_module=v1_components_retrieve,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Component))},
    },
)
@click.pass_context
@progress_bar
def get_component(ctx, component_id, purl, **params):
    is_params_empty = [False for v in params.values() if v]
    if not component_id and not purl and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)

    if purl:
        params["purl"] = purl

    if not params["include_fields"]:
        params[
            "include_fields"
        ] = "link,purl,nvr,version,type,name,upstreams,related_url,download_url"

    params = multivalue_params_to_csv(params)

    session = CommunityComponentService.create_session()
    if component_id:
        data = session.components.retrieve(component_id, **params)
    else:
        data = session.components.retrieve_list(**params)
    return cprint(data, ctx=ctx)


@components.command(
    name="summary",
    help="Get Component summaries.",
)
@click.argument(
    "component_name",
    required=False,
)
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of name.",
)
@query_params_options(
    entity="Component",
    endpoint_module=v1_components_list,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Component))},
    },
)
@click.pass_context
@progress_bar
def get_component_summary(ctx, component_name, strict_name_search, **params):
    """Get Component summary."""
    is_params_empty = [False for v in params.values() if v]
    if not component_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()

    cond = {
        "include_fields": "name,type,download_url,purl,tags,arch,release,version,product_streams,upstreams,related_url",  # noqa
        "name": component_name,
    }
    components = session.components.retrieve_list(**cond, limit=10000)
    product_streams = []
    upstreams = []
    versions = []
    releases = []
    arches = []
    related_urls = []
    download_urls = []
    tags = []
    component_type = None
    for component in components.results:
        component_type = component.type
        related_urls.append(component.related_url)
        arches.append(component.arch)
        versions.append(component.version)
        releases.append(component.release)
        tags.extend(component.tags)
        for upstream in component.upstreams:
            upstreams.append(upstream["purl"])
        for ps in component.product_streams:
            product_streams.append(ps["name"])

    cond = {
        "include_fields": "name,purl,version,type,tags,arch,release,product_streams",  # noqa
        "name": component_name,
        "view": "latest",
    }
    latest_components = session.components.retrieve_list(**cond, limit=10000)

    latest = []

    for latest_component in latest_components.results:
        latest.append(
            {
                "product_stream": latest_component["product_stream"],
                "purl": latest_component.purl,
            }
        )
    data = {
        "link": f"{CORGI_API_URL}/api/v1/components?name={component_name}",
        "type": component_type,
        "name": component_name,
        "tags": sorted(list(set(tags))),
        "count": len(components.results),
        "product_streams": sorted(list(set(product_streams))),
        "related_urls": sorted(list(set(related_urls))),
        "download_urls": sorted(list(set(download_urls))),
        "releases": sorted(list(set(releases))),
        "upstreams": sorted(list(set(upstreams))),
        "arches": sorted(list(set(arches))),
        "versions": sorted(list(set(versions))),
        "latest": latest,
    }
    cprint(data, ctx=ctx)


@components.command(name="provides")
@click.option("--uuid", "component_uuid")
@click.option("--purl", shell_complete=get_component_purls, help="Purl are URI and must be quoted.")
@query_params_options(
    entity="Component",
    endpoint_module=v1_components_list,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Component))},
    },
)
@click.pass_context
@progress_bar
def get_component_provides(ctx, component_uuid, purl, **params):
    """Retrieve all Components provided by a Component."""
    is_params_empty = [False for v in params.values() if v]
    if not component_uuid and not purl and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    if not params["include_fields"]:
        params[
            "include_fields"
        ] = "link,purl,nvr,version,type,name,upstreams,related_url,download_url"
    if purl:
        params["sources"] = purl

    session = CommunityComponentService.create_session()
    if component_uuid:
        purl = session.components.retrieve(component_uuid).purl
        params["sources"] = purl
        data = session.components.retrieve_list(**params)
        return cprint(data, ctx=ctx)
    else:
        data = session.components.retrieve_list(**params)
        return cprint(data, ctx=ctx)


@components.command(name="sources")
@click.option("--uuid", "component_uuid")
@click.option("--purl", shell_complete=get_component_purls, help="Purl are URI and must be quoted.")
@query_params_options(
    entity="Component",
    endpoint_module=v1_components_list,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Component))},
    },
)
@click.pass_context
@progress_bar
def get_component_sources(ctx, component_uuid, purl, **params):
    """Retrieve all Components that contain Component."""
    is_params_empty = [False for v in params.values() if v]
    if not component_uuid and not purl and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    if not params["include_fields"]:
        params[
            "include_fields"
        ] = "link,purl,nvr,version,type,name,upstreams,related_url,download_url"
    if purl:
        params["provides"] = purl
    session = CommunityComponentService.create_session()
    if component_uuid:
        purl = session.components.retrieve(component_uuid).purl
        params["provides"] = purl
        data = session.components.retrieve_list(**params)
        return cprint(data, ctx=ctx)
    else:
        data = session.components.retrieve_list(**params)
        return cprint(data, ctx=ctx)


@components.command(name="manifest")
@click.option("--uuid", "component_uuid")
@click.option("--purl", shell_complete=get_component_purls, help="Purl are URI and must be quoted.")
@click.option(
    "--spdx-json",
    "spdx_json_format",
    is_flag=True,
    default=False,
    help="Generate spdx manifest (json).",
)
@click.pass_context
@progress_bar
def get_component_manifest(ctx, component_uuid, purl, spdx_json_format):
    """Retrieve Component manifest."""
    if not component_uuid and not purl:
        click.echo(ctx.get_help())
        exit(0)
    if spdx_json_format:
        ctx.ensure_object(dict)
        ctx.obj["FORMAT"] = "json"  # TODO - investigate if we need yaml format.
    session = CommunityComponentService.create_session()
    if component_uuid:
        data = session.components.retrieve_manifest(component_uuid)
        return cprint(data, ctx=ctx)
    else:
        c = session.components.retrieve_list(purl=purl)
        if c:
            data = session.components.retrieve_manifest(c["uuid"])
            return cprint(data, ctx=ctx)


# PRODUCT STREAM
@commmunity_components_grp.group(help=f"{CORGI_API_URL}/api/v1/product_streams")
@click.pass_context
def product_streams(ctx):
    pass


@product_streams.command(name="list")
@click.argument(
    "product_stream_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@query_params_options(
    entity="ProductStream",
    endpoint_module=v1_product_streams_list,
    options_overrides={
        "include_fields": {
            "type": click.Choice(CommunityComponentService.get_fields(ProductStream))
        },
    },
)
@click.pass_context
def list_product_streams(ctx, product_stream_name, **params):
    """Retrieve a list of Product Streams."""
    is_params_empty = [False for v in params.values() if v]
    if not product_stream_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()
    if not params["include_fields"]:
        params["include_fields"] = "link,uuid,ofuri,name"

    if product_stream_name:
        params["re_name"] = product_stream_name
    data = session.product_streams.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@product_streams.command(name="get")
@click.argument(
    "product_stream_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@click.option("--inactive", is_flag=True, default=False, help="Show inactive project streams")
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
@query_params_options(
    entity="ProductStream",
    endpoint_module=v1_product_streams_retrieve,
    options_overrides={
        "include_fields": {
            "type": click.Choice(CommunityComponentService.get_fields(ProductStream))
        },
    },
)
@click.pass_context
@progress_bar
def get_product_stream(ctx, product_stream_name, inactive, ofuri, **params):
    """Retrieve Product Stream."""
    is_params_empty = [False for v in params.values() if v]
    if not product_stream_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()
    if ofuri:
        params["ofuri"] = ofuri
    if product_stream_name:
        params["name"] = product_stream_name
    data = session.product_streams.retrieve_list(**params)
    return cprint(data, ctx=ctx)


@product_streams.command(name="latest-components")
@click.argument(
    "product_stream_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
@query_params_options(
    entity="Component",
    endpoint_module=v1_components_list,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Component))},
    },
)
@click.pass_context
def get_product_stream_components(ctx, product_stream_name, ofuri, **params):
    """Retrieve Product Stream latest Components."""
    if not ofuri and not product_stream_name:
        click.echo(ctx.get_help())
        exit(0)
    if not params["include_fields"]:
        params[
            "include_fields"
        ] = "link,uuid,purl,nvr,version,type,name,upstreams,related_url,download_url"

    if product_stream_name:
        session = CommunityComponentService.create_session()
        ps = session.product_streams.retrieve_list(name=product_stream_name)
        params["ofuri"] = ps["ofuri"]
    if ofuri:
        params["ofuri"] = ofuri
    ctx.invoke(list_components, **params)


@product_streams.command(name="manifest")
@click.argument(
    "product_stream_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
@click.option(
    "--spdx-json",
    "spdx_json_format",
    is_flag=True,
    default=False,
    help="Generate spdx manifest (json).",
)
@click.pass_context
@progress_bar
def get_product_stream_manifest(ctx, product_stream_name, ofuri, spdx_json_format):
    """Retrieve Product Stream manifest."""
    if not ofuri and not product_stream_name:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()
    if spdx_json_format:
        ctx.ensure_object(dict)
        ctx.obj["FORMAT"] = "json"  # TODO - investigate if we need yaml format.
    ps = None
    if ofuri:
        ps = session.product_streams.retrieve_list(ofuri=ofuri).additional_properties
    if product_stream_name:
        ps = session.product_streams.retrieve_list(name=product_stream_name).additional_properties
    if not ps:
        logger.warning("could not find active product stream.")
    data = requests.get(ps["manifest"])
    cprint(data.json(), ctx=ctx)


# BUILDS


@commmunity_components_grp.group(help=f"{CORGI_API_URL}/api/v1/builds")
@click.pass_context
def builds(ctx):
    pass


@builds.command(name="list")
@click.argument("software_build_name", required=False)
@query_params_options(
    entity="SoftwareBuild",
    endpoint_module=v1_builds_list,
    options_overrides={
        "include_fields": {
            "type": click.Choice(CommunityComponentService.get_fields(SoftwareBuild))
        },
    },
)
@click.pass_context
def list_software_builds(ctx, software_build_name, **params):
    """Retrieve a list of Software Builds."""
    session = CommunityComponentService.create_session()
    if software_build_name:
        params["name"] = software_build_name
    data = session.builds.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@builds.command(name="get")
@click.argument(
    "software_build_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@query_params_options(
    entity="SoftwareBuild",
    endpoint_module=v1_builds_retrieve,
    options_overrides={
        "include_fields": {
            "type": click.Choice(CommunityComponentService.get_fields(SoftwareBuild))
        },
    },
)
@click.pass_context
@progress_bar
def get_software_build(ctx, software_build_name, **params):
    """Retrieve SoftwareBuild."""
    is_params_empty = [False for v in params.values() if v]
    if not software_build_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()
    if software_build_name:
        params["name"] = software_build_name
    data = session.builds.retrieve_list(**params)
    return cprint(data, ctx=ctx)


# Products


@commmunity_components_grp.group(help=f"{CORGI_API_URL}/api/v1/products")
@click.pass_context
def products(ctx):
    pass


@products.command(name="list")
@click.argument("product_name", required=False)
@query_params_options(
    entity="Product",
    endpoint_module=v1_products_list,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Product))},
    },
)
@click.pass_context
@progress_bar
def list_products(ctx, product_name, **params):
    """Retrieve a list of Software Builds."""
    session = CommunityComponentService.create_session()
    if product_name:
        params["re_name"] = product_name
    data = session.products.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@products.command(name="get")
@click.argument(
    "product_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
@query_params_options(
    entity="Product",
    endpoint_module=v1_products_retrieve,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Product))},
    },
)
@click.pass_context
@progress_bar
def get_product(ctx, product_name, ofuri, **params):
    """Retrieve Product."""
    is_params_empty = [False for v in params.values() if v]
    if not product_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()
    if ofuri:
        params["ofuri"] = ofuri
    if product_name:
        params["name"] = product_name
    data = session.products.retrieve_list(**params)
    return cprint(data, ctx=ctx)


# PRODUCT VERSION
@commmunity_components_grp.group(help=f"{CORGI_API_URL}/api/v1/product-versions")
@click.pass_context
def product_versions(ctx):
    pass


@product_versions.command(name="list")
@click.argument("product_version_name", required=False)
@query_params_options(
    entity="ProductVersion",
    endpoint_module=v1_product_versions_list,
    options_overrides={
        "include_fields": {
            "type": click.Choice(CommunityComponentService.get_fields(ProductVersion))
        },
    },
)
@click.pass_context
@progress_bar
def list_product_versions(ctx, product_version_name, **params):
    """Retrieve a list of Product Versions."""
    session = CommunityComponentService.create_session()
    if product_version_name:
        params["re_name"] = product_version_name
    data = session.product_versions.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@product_versions.command(name="get")
@click.argument(
    "product_version_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_version_ofuris)
@query_params_options(
    entity="ProductVersion",
    endpoint_module=v1_product_versions_retrieve,
    options_overrides={
        "include_fields": {
            "type": click.Choice(CommunityComponentService.get_fields(ProductVersion))
        },
    },
)
@click.pass_context
@progress_bar
def get_product_version(ctx, product_version_name, ofuri, **params):
    """Retrieve ProductVersion."""
    is_params_empty = [False for v in params.values() if v]
    if not product_version_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()
    if ofuri:
        params["ofuri"] = ofuri
    if product_version_name:
        params["name"] = product_version_name
    data = session.product_versions.retrieve_list(**params)
    return cprint(data, ctx=ctx)


# PRODUCT VARIANT
@commmunity_components_grp.group(help=f"{CORGI_API_URL}/api/v1/product-variants")
@click.pass_context
def product_variants(ctx):
    pass


@product_variants.command(name="list")
@click.argument("product_variant_name", required=False)
@query_params_options(
    entity="ProductVariant",
    endpoint_module=v1_product_variants_list,
    options_overrides={
        "include_fields": {
            "type": click.Choice(CommunityComponentService.get_fields(ProductVariant))
        },
    },
)
@click.pass_context
@progress_bar
def list_product_variants(ctx, product_variant_name, **params):
    """Retrieve a list of Product Variants."""
    session = CommunityComponentService.create_session()
    if product_variant_name:
        params["re_name"] = product_variant_name
    data = session.product_variants.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@product_variants.command(name="get")
@click.argument(
    "product_variant_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_version_ofuris)
@query_params_options(
    entity="ProductVariant",
    endpoint_module=v1_product_variants_retrieve,
    options_overrides={
        "include_fields": {
            "type": click.Choice(CommunityComponentService.get_fields(ProductVariant))
        },
    },
)
@click.pass_context
@progress_bar
def get_product_variant(ctx, product_variant_name, ofuri, **params):
    """Retrieve ProductVariant."""
    is_params_empty = [False for v in params.values() if v]
    if not product_variant_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()
    if ofuri:
        params["ofuri"] = ofuri
    if product_variant_name:
        params["name"] = product_variant_name
    data = session.product_varints.retrieve_list(**params)
    return cprint(data, ctx=ctx)


# CHANNEL
@commmunity_components_grp.group(help=f"{CORGI_API_URL}/api/v1/channels")
@click.pass_context
def channels(ctx):
    pass


@channels.command(name="list")
@click.argument("channel_name", required=False)
@query_params_options(
    entity="Channel",
    endpoint_module=v1_channels_list,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Channel))},
    },
)
@click.pass_context
@progress_bar
def list_channels(ctx, channel_name, **params):
    """Retrieve a list of Channels."""
    session = CommunityComponentService.create_session()
    if channel_name:
        params["re_name"] = channel_name
    data = session.channels.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@channels.command(name="get")
@click.argument(
    "channel_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_version_ofuris)
@query_params_options(
    entity="Channel",
    endpoint_module=v1_channels_retrieve,
    options_overrides={
        "include_fields": {"type": click.Choice(CommunityComponentService.get_fields(Channel))},
    },
)
@click.pass_context
@progress_bar
def get_channel(ctx, channel_name, ofuri, **params):
    """Retrieve ProductVariant."""
    is_params_empty = [False for v in params.values() if v]
    if not channel_name and not is_params_empty:
        click.echo(ctx.get_help())
        exit(0)
    session = CommunityComponentService.create_session()
    if ofuri:
        params["ofuri"] = ofuri
    if channel_name:
        params["name"] = channel_name
    data = session.channels.retrieve_list(**params)
    return cprint(data, ctx=ctx)


# ADMIN
@commmunity_components_grp.group(name="admin")
@click.pass_context
def manage_grp(ctx):
    """Manage component registry"""
    pass


@manage_grp.command(name="status")
@click.pass_context
def corgi_status(ctx):
    session = CommunityComponentService.create_session()
    data = session.status()
    return cprint(data.additional_properties, ctx=ctx)


@manage_grp.command(name="health")
@click.pass_context
def corgi_health(ctx):
    try:
        session = CommunityComponentService.create_session()
        status = session.status()["status"]
        if status == "ok":
            console.log(f"{CORGI_API_URL} is operational")
        else:
            console.log(f"{CORGI_API_URL} is NOT operational")
            exit(1)
    except:  # noqa
        console.log(f"{CORGI_API_URL} is NOT operational")
        raise click.ClickException("Component registry health check failed.")


@manage_grp.command(name="data")
def corgi_data():
    click.launch(f"{CORGI_API_URL}/data")


@manage_grp.command(name="api_doc")
def corgi_api_docs():
    click.launch(f"{CORGI_API_URL}/api/v1/schema/docs")



import enum
import json
import logging

import click
from packageurl import PackageURL
from rich.console import Console
from rich.text import Text
from rich.tree import Tree

console = Console(color_system="auto")

logger = logging.getLogger("griffon")


class OUTPUT_FORMAT(enum.Enum):
    JSON = "json"
    TEXT = "text"
    TABLE = "table"


class DEST(enum.Enum):
    CONSOLE = "console"
    FILE = "file"


def raw_json_transform(data, show_count: bool) -> dict:
    """normalise all data to dict"""
    if type(data) is list:
        results = []
        for d in data:
            if type(d) is dict:
                results.append(d)
            else:
                results.append(d.to_dict())
        output = {
            "results": results,
        }
        if show_count:
            output["count"] = len(results)  # type: ignore
    else:
        if type(data) is dict:
            output = data
        else:
            output = data.to_dict()
    return output


def entity_type(data):
    entity_type = "unknown"
    if "cve_id" in data:
        entity_type = "flaw"
    if "purl" in data:
        entity_type = "component"
    if "affectedness" in data:
        entity_type = "affect"
    if "ofuri" in data:
        entity_type = "product"
    return entity_type


def component_type_style(type):
    from griffon import CorgiService

    types = [type.value for type in CorgiService.get_component_types()]
    colors = (
        "cornflower_blue",
        "red",
        "dark_slate_gray1",
        "magenta",
        "navy_blue",
        "green1",
        "blue",
        "dark_orange",
        "deep_pink1",
        "red1",
        "green1",
    )
    color = colors[types.index(type)]
    return f"[{color}]{type}[/{color}]"


def output_version(ctx, version):
    if version:
        if version.startswith("sha256") and ctx.obj["SHORT_VERSION_VALUES"]:
            return f"sha256...{version[-8:]}"
    return version


def walk_component_tree(obj, key, tree, show_purl=False):
    if isinstance(obj, dict):
        for k, v in obj.items():
            if isinstance(v, (dict, list)):
                walk_component_tree(v, k, tree, show_purl=show_purl)
    elif isinstance(obj, list):
        for item in obj:
            label = f"{item['node_type']}: {item['nvr']},{item['type']}"
            if show_purl:
                label = f"{item['node_type']}: {item['purl']}"
            child = tree.add(label)
            walk_component_tree(item, None, child, show_purl=show_purl)
    return tree


def text_output_tree(ctx, output):
    tree = Tree(
        "component dependency tree",
        guide_style="bold magenta",
    )
    console.print(walk_component_tree(output, None, tree, show_purl=ctx.params["show_purl"]))
    ctx.exit()


def text_output_product_summary(ctx, output, format, exclude_products):
    ordered_results = sorted(output["results"], key=lambda d: d["name"])

    if exclude_products:
        exclude_products_results = []
        for result in ordered_results:
            if result["product_version"] not in exclude_products:
                exclude_products_results.append(result)
        ordered_results = exclude_products_results

    if ctx.obj["VERBOSE"] == 0:
        for item in ordered_results:
            console.print(
                Text(item["product"], style="bold magenta u"),
                Text(item["product_version"], style="magenta"),
                Text(item["name"], style="white"),
                f"{item['brew_tags']}",
                no_wrap=False,
            )
    if ctx.obj["VERBOSE"] == 1:
        for item in ordered_results:
            console.print(
                Text(item["product"], style="bold magenta u"),
                Text(item["product_version"], style="magenta"),
                Text(item["name"], style="white"),
                item["brew_tags"],
                item["ofuri"],
                no_wrap=False,
            )
    if ctx.obj["VERBOSE"] > 1:
        for item in ordered_results:
            console.print(
                Text(item["product"], style="bold magenta u"),
                Text(item["product_version"], style="magenta"),
                Text(item["name"], style="white"),
                item["brew_tags"],
                item["ofuri"],
                item["manifest_link"],
                no_wrap=False,
            )
    ctx.exit()


def text_output_products_contain_component(
    ctx, output, format, exclude_products, exclude_components
):
    component_name = ctx.params["component_name"]

    # handle single component
    if ctx.params["purl"]:
        ordered_results = sorted(output["results"], key=lambda d: d["ofuri"])
        for item in ordered_results:
            console.print(
                Text(item["ofuri"], style="bold magenta u"),
                no_wrap=False,
            )
        ctx.exit()

    # handle multiple components
    if "results" in output and output["count"] > 0:
        ordered_results = sorted(output["results"], key=lambda d: d["product_stream"])

        if exclude_products:
            exclude_products_results = []
            for result in ordered_results:
                if result["product_version"] not in exclude_products:
                    exclude_products_results.append(result)
            ordered_results = exclude_products_results

        # TODO - MAVEN component type will require special handling
        if ctx.params["affect_mode"]:
            console.no_color = True
            console.highlighter = None

            flaw_mode = ctx.params["flaw_mode"]
            flaw_operation = "dry_run"
            if flaw_mode == "add":
                flaw_operation = "new"
            if flaw_mode == "update":
                flaw_operation = "update"
            product_versions = sorted(
                list(set([item["product_version"] for item in ordered_results]))
            )
            for pv in product_versions:
                names = [item["name"] for item in ordered_results if pv == item["product_version"]]
                names = list(set(names))
                if component_name in names and f"{component_name}-container" in names:
                    names.remove(f"{component_name}-container")
                for name in names:
                    if not any([match in name for match in exclude_components]):
                        dep_name = name
                        console.print(
                            f"{pv}/{name}={flaw_operation}",
                            no_wrap=False,
                        )
        else:
            if ctx.obj["VERBOSE"] == 0:  # product_version X source component
                product_versions = sorted(
                    list(set([item["product_version"] for item in ordered_results]))
                )
                for pv in product_versions:
                    names = [
                        item["name"] for item in ordered_results if pv == item["product_version"]
                    ]
                    names = list(set(names))
                    if component_name in names and f"{component_name}-container" in names:
                        names.remove(f"{component_name}-container")
                    for name in names:
                        if not any([match in name for match in exclude_components]):
                            dep_name = name.replace(component_name, f"[b]{component_name}[/b]")
                            dep = f"[white]({dep_name})[/white]"
                            console.print(
                                Text(pv, style="magenta b u"),
                                dep,
                                no_wrap=False,
                            )

            if ctx.obj["VERBOSE"] == 1:  # product_stream X source component
                product_streams = sorted(
                    list(set([item["product_stream"] for item in ordered_results]))
                )
                for ps in product_streams:
                    ps_components = [
                        item for item in ordered_results if item["product_stream"] == ps
                    ]
                    names = sorted(list(set([item["name"] for item in ps_components])))
                    if component_name in names and f"{component_name}-container" in names:
                        names.remove(f"{component_name}-container")

                    for name in names:
                        if not any([match in name for match in exclude_components]):
                            sources = []
                            nvr = ""
                            for item in ps_components:
                                if item["name"] == name and "sources" in item:
                                    sources.extend(item["sources"])
                                if item["nvr"]:
                                    nvr = item["nvr"]

                            root_component = "root component"
                            if sources:
                                source_purl = PackageURL.from_string(sources[0]["purl"])
                                root_component = source_purl.name

                            dep_name = nvr.replace(component_name, f"[b]{component_name}[/b]")
                            dep = f"[white]({dep_name}, {item['type'].lower()})[/white]"
                            console.print(
                                Text(ps, style="magenta b u"),
                                root_component,
                                dep,
                                no_wrap=False,
                            )

            if ctx.obj["VERBOSE"] == 2:  # product_stream X nvr
                product_streams = sorted(
                    list(set([item["product_stream"] for item in ordered_results]))
                )
                for ps in product_streams:
                    ps_components = [
                        item for item in ordered_results if item["product_stream"] == ps
                    ]
                    names = sorted(list(set([item["name"] for item in ps_components])))
                    if component_name in names and f"{component_name}-container" in names:
                        names.remove(f"{component_name}-container")

                    for name in names:
                        if not any([match in name for match in exclude_components]):
                            sources = []
                            nvr = ""
                            for item in ps_components:
                                if item["name"] == name and "sources" in item:
                                    sources.extend(item["sources"])
                                if item["nvr"]:
                                    nvr = item["nvr"]

                            root_component = "root component"
                            if sources:
                                source_purl = PackageURL.from_string(sources[0]["purl"])
                                root_component = (
                                    f"{source_purl.name}-{output_version(ctx,source_purl.version)}"
                                )

                            dep_name = nvr.replace(component_name, f"[b]{component_name}[/b]")
                            dep = f"[white]({dep_name}, {item['type'].lower()})[/white]"
                            console.print(
                                Text(ps, style="magenta b u"),
                                root_component,
                                dep,
                                no_wrap=False,
                            )

            if ctx.obj["VERBOSE"] == 3:  # source url, upstream
                product_streams = sorted(
                    list(set([item["product_stream"] for item in ordered_results]))
                )
                for ps in product_streams:
                    ps_components = [
                        item for item in ordered_results if item["product_stream"] == ps
                    ]
                    names = sorted(list(set([item["name"] for item in ps_components])))
                    if component_name in names and f"{component_name}-container" in names:
                        names.remove(f"{component_name}-container")
                    for name in names:
                        if not any([match in name for match in exclude_components]):
                            sources = []
                            related_url = ""
                            build_source_url = ""
                            nvr = ""
                            for item in ps_components:
                                if item["name"] == name:
                                    if item["nvr"]:
                                        nvr = item["nvr"]
                                    if "sources" in item:
                                        sources.extend(item["sources"])
                                    if item["related_url"]:
                                        related_url = item["related_url"]
                                    if item["build_source_url"]:
                                        build_source_url = item["build_source_url"]
                            root_component = "root component"
                            if sources:
                                source_purl = PackageURL.from_string(sources[0]["purl"])
                                root_component = (
                                    f"{source_purl.name}-{output_version(ctx,source_purl.version)}"
                                )
                            dep_name = nvr.replace(component_name, f"[b]{component_name}[/b]")
                            dep = f"[white]({dep_name}, {item['type'].lower()})[/white]"
                            related_url = related_url.replace(
                                component_name, f"[b]{component_name}[/b]"
                            )
                            console.print(
                                Text(ps, style="magenta b u"),
                                root_component,
                                dep,
                                related_url,
                                no_wrap=False,
                            )
            if ctx.obj["VERBOSE"] > 3:  # source url, upstream
                product_streams = sorted(
                    list(set([item["product_stream"] for item in ordered_results]))
                )
                for ps in product_streams:
                    ps_components = [
                        item for item in ordered_results if item["product_stream"] == ps
                    ]
                    names = sorted(list(set([item["name"] for item in ps_components])))
                    if component_name in names and f"{component_name}-container" in names:
                        names.remove(f"{component_name}-container")
                    for name in names:
                        if not any([match in name for match in exclude_components]):
                            sources = []
                            related_url = ""
                            build_source_url = ""
                            nvr = ""
                            for item in ps_components:
                                if item["name"] == name:
                                    if item["nvr"]:
                                        nvr = item["nvr"]
                                    if "sources" in item:
                                        sources.extend(item["sources"])
                                    if item["related_url"]:
                                        related_url = item["related_url"]
                                    if item["build_source_url"]:
                                        build_source_url = item["build_source_url"]
                            root_component = "root component"
                            if sources:
                                source_purl = PackageURL.from_string(sources[0]["purl"])
                                root_component = (
                                    f"{source_purl.name}-{output_version(ctx,source_purl.version)}"
                                )
                            upstream = ""
                            if item["upstream_purl"]:
                                upstream = f"[cyan]{item['upstream_purl']}[/cyan]"
                            dep_name = nvr.replace(component_name, f"[b]{component_name}[/b]")
                            dep = f"[white]({dep_name}, {item['type'].lower()})[/white]"
                            related_url = related_url.replace(
                                component_name, f"[b]{component_name}[/b]"
                            )
                            console.print(
                                Text(ps, style="magenta b u"),
                                root_component,
                                dep,
                                related_url,
                                build_source_url,
                                upstream,
                                no_wrap=False,
                            )

        ctx.exit()


def text_output_components_contain_component(ctx, output, format, exclude_components):
    if "results" in output:
        for item in output["results"]:
            component_name = item["name"]
            component_nvr = item["nvr"]
            related_url = item["related_url"]
            download_url = item["download_url"]
            arch = item["arch"]
            if not any([match in component_name for match in exclude_components]):
                if ctx.obj["VERBOSE"] == 0:
                    ordered_sources = sorted(item["sources"], key=lambda d: d["purl"])
                    for source in ordered_sources:
                        if "arch=noarch" in source["purl"] or "arch=src" in source["purl"]:
                            source_purl = PackageURL.from_string(source["purl"])
                            root_component = source_purl.name
                            if source_purl.type == "oci" and "-source" not in source_purl.name:
                                root_component = (
                                    f"[u magenta]{source_purl.name}-container[/u magenta]"
                                )
                            if source_purl.type == "oci":
                                component_ns = Text("REDHAT", style="bold magenta")
                            elif not source_purl.namespace:
                                component_ns = Text("UPSTREAM", style="bold magenta")
                            else:
                                component_ns = Text(source_purl.namespace.upper(), style="bold red")

                            console.print(
                                component_ns,
                                source_purl.type.upper(),
                                root_component,
                                Text(component_name, style="bold white"),
                                no_wrap=False,
                            )
                if ctx.obj["VERBOSE"] == 1:
                    ordered_sources = sorted(item["sources"], key=lambda d: d["purl"])
                    for source in ordered_sources:
                        if "arch=noarch" in source["purl"] or "arch=src" in source["purl"]:
                            source_purl = PackageURL.from_string(source["purl"])
                            root_component = f"{source_purl.name}-{source_purl.version}"
                            if source_purl.type == "oci" and "-source" not in source_purl.name:
                                root_component = (
                                    f"[u magenta]{source_purl.name}-container[/u magenta]"
                                )
                            if source_purl.type == "oci":
                                component_ns = Text("REDHAT", style="bold magenta")
                            elif not source_purl.namespace:
                                component_ns = Text("UPSTREAM", style="bold magenta")
                            else:
                                component_ns = Text(source_purl.namespace.upper(), style="bold red")
                            console.print(
                                component_ns,
                                source_purl.type.upper(),
                                root_component,
                                Text(component_nvr, style="bold white"),
                                arch,
                                no_wrap=False,
                            )
                if ctx.obj["VERBOSE"] > 1:
                    ordered_sources = sorted(item["sources"], key=lambda d: d["purl"])
                    for source in ordered_sources:
                        if "arch=noarch" in source["purl"] or "arch=src" in source["purl"]:
                            source_purl = PackageURL.from_string(source["purl"])
                            root_component = f"{source_purl.name}-{source_purl.version}"
                            if source_purl.type == "oci" and "-source" not in source_purl.name:
                                root_component = (
                                    f"[u magenta]{source_purl.name}-container[/u magenta]"
                                )
                            if source_purl.type == "oci":
                                component_ns = Text("REDHAT", style="bold magenta")
                            elif not source_purl.namespace:
                                component_ns = Text("UPSTREAM", style="bold magenta")
                            else:
                                component_ns = Text(source_purl.namespace.upper(), style="bold red")
                            console.print(
                                component_ns,
                                source_purl.type.upper(),
                                root_component,
                                Text(component_nvr, style="bold white"),
                                arch,
                                related_url,
                                download_url,
                                no_wrap=False,
                            )
    ctx.exit()


def text_output_components_affected_by_cve(ctx, output, format):
    console.print("Flaw Title:", output["title"])
    console.print(
        "affects:",
    )
    for component in output["components"]:
        affected_component1 = f"({component['purl']})"
        if not ctx.obj["SHOW_PURL"]:
            purl = PackageURL.from_string(component["purl"])
            ns = "UPSTREAM"
            if purl.namespace:
                ns = purl.namespace.upper()
            affected_component = f"([bold cyan]{ns}[/bold cyan] {purl.name}-{purl.version},{purl.type.upper()})"  # noqa
            if ctx.obj["VERBOSE"] == 0:
                console.print(
                    Text(component["product_version"], style="bold magenta u"),
                    ns,
                    affected_component1,
                    no_wrap=False,
                )
            if ctx.obj["VERBOSE"] == 1:
                console.print(
                    Text(component["product_stream"], style="bold magenta u"),
                    ns,
                    affected_component1,
                    no_wrap=False,
                )
            if ctx.obj["VERBOSE"] > 1:
                console.print(
                    Text(component["product_stream"], style="bold magenta u"),
                    ns,
                    affected_component1,
                    Text(component["build_source_url"], style="i"),
                    Text(component["related_url"], style="i"),
                    Text(component["download_url"], style="i"),
                    no_wrap=False,
                )
    ctx.exit()


def text_output_products_affected_by_cve(ctx, output, format, exclude_products):
    console.print("[white]link:[/white]", output["link"])
    console.print("[white]cve_id:[/white]", output["cve_id"])
    console.print("[white]title:[/white]", output["title"])
    if ctx.obj["VERBOSE"] == 0:
        console.print(
            "[white]product_versions:[/white]",
        )
        ordered_product_versions = sorted(output["product_versions"])
        for product_version in ordered_product_versions:
            console.print(Text(product_version, style="bold magenta u"), no_wrap=True)
    if ctx.obj["VERBOSE"] > 0:
        console.print(
            "[white]product_streams:[/white]",
        )
        ordered_product_streams = sorted(output["product_streams"])
        for product_stream in ordered_product_streams:
            console.print(Text(product_stream, style="bold magenta u"), no_wrap=True)
    ctx.exit()


def text_output_get_manifest(ctx, output, format):
    if not ctx.obj["SHOW_PURL"]:
        for component in output["packages"]:
            if "pkg:" in component["externalRefs"][0]["referenceLocator"]:
                purl = PackageURL.from_string(component["externalRefs"][0]["referenceLocator"])
                ns = "[cyan]UPSTREAM[/cyan]"
                component = f"([bold turquoise2]{ns}[/bold turquoise2] [white]{purl.name}-{purl.version}[/white],{component_type_style(purl.type.upper())})"  # noqa
                if purl.namespace == "redhat":
                    ns = f"[red]{purl.namespace.upper()}[/red]"
                    component = f"([white]{purl.name}-{purl.version}[/white],{component_type_style(purl.type.upper())})"  # noqa
                else:
                    if purl.namespace:
                        ns = f"[white]{purl.namespace.upper()}[/white]"
                    component = f"([white]{purl.name}-{purl.version}[/white],{component_type_style(purl.type.upper())})"  # noqa
                console.print(ns, component, no_wrap=False)  # noqa
    else:
        for component in output["packages"]:
            purl = component["externalRefs"][0]["referenceLocator"]
            console.print(purl, no_wrap=False)  # noqa

    ctx.exit()


def text_output_component_flaws(ctx, output, format):
    ordered_components = sorted(output["results"], key=lambda d: d["name"])
    for item in ordered_components:
        component_name = item["name"]
        # sorting should work when there is no title or cve-id key
        ordered_affects = sorted(item["affects"], key=lambda d: (d["title"] is None, d["title"]))
        for affect in ordered_affects:
            flaw_cve_id = "Vulnerability"
            if affect["flaw_cve_id"]:
                flaw_cve_id = affect["flaw_cve_id"]
            if ctx.obj["VERBOSE"] == 0:
                console.print(
                    Text(flaw_cve_id, style="magenta"),
                    Text(component_name, style="white"),
                    Text(affect["affect_product_version"], style="cyan"),
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
            if ctx.obj["VERBOSE"] == 1:
                console.print(
                    Text(flaw_cve_id, style="magenta"),
                    Text(component_name, style="white"),
                    Text(affect["affect_product_version"], style="cyan"),
                    f"(state: {affect['flaw_state']} resolution:{affect['flaw_resolution']})",
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
            if ctx.obj["VERBOSE"] > 1:
                console.print(Text(affect["title"], style="white"))
                console.print(
                    Text(component_name, style="magenta"),
                    Text(affect["affect_product_version"], style="cyan"),
                    f"(state: {affect['flaw_state']} resolution:{affect['flaw_resolution']})",
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
    ctx.exit()


def text_output_product_flaws(ctx, output, format):
    for item in output["results"]:
        component_name = item["name"]
        for affect in item["affects"]:
            flaw_cve_id = "Vulnerability"
            if affect["flaw_cve_id"]:
                flaw_cve_id = affect["flaw_cve_id"]
            if ctx.obj["VERBOSE"] == 0:
                console.print(
                    Text(flaw_cve_id, style="magenta"),
                    Text(component_name, style="white"),
                    affect["affect_component_name"],
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
            if ctx.obj["VERBOSE"] == 1:
                console.print(
                    Text(flaw_cve_id, style="magenta"),
                    Text(component_name, style="white"),
                    f"(state: {affect['flaw_state']} resolution:{affect['flaw_resolution']})",
                    affect["affect_component_name"],
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
            if ctx.obj["VERBOSE"] > 1:
                console.print(affect["title"])
                console.print(
                    Text(flaw_cve_id, style="magenta"),
                    Text(component_name, style="white"),
                    f"(state: {affect['flaw_state']} resolution:{affect['flaw_resolution']})",
                    affect["affect_component_name"],
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
    ctx.exit()


def text_output_list(ctx, output, format, exclude_components):
    if "results" in output and output["count"] > 0:
        # handle component
        if "purl" in output["results"][0]:
            ordered_components = sorted(output["results"], key=lambda d: d["name"])
            for row in ordered_components:
                if not any([match in row["purl"] for match in exclude_components]):
                    if "purl" in row:
                        purl = PackageURL.from_string(row["purl"])
                        if purl.type == "oci":
                            component_ns = Text("REDHAT", style="bold magenta")
                        elif not purl.namespace:
                            component_ns = Text("UPSTREAM", style="bold magenta")
                        else:
                            component_ns = Text(purl.namespace.upper(), style="bold red")

                        sha256 = ""
                        if purl.version:
                            if purl.version.startswith("sha256"):
                                sha256 = output_version(ctx, purl.version)
                        nvr = None
                        if "nvr" in row:
                            nvr = row["nvr"]

                        if not ctx.obj["SHOW_PURL"]:
                            if ctx.obj["VERBOSE"] == 0:
                                console.print(
                                    component_ns,
                                    purl.type.upper(),
                                    Text(nvr, style="bold white"),
                                    sha256,
                                    row["related_url"],
                                    purl.qualifiers.get("arch"),
                                )
                            if ctx.obj["VERBOSE"] > 0:
                                download_url = ""
                                if "download_url" in row:
                                    download_url = row["download_url"]
                                console.print(
                                    component_ns,
                                    purl.type.upper(),
                                    Text(nvr, style="bold white"),
                                    sha256,
                                    row["related_url"],
                                    purl.qualifiers.get("arch"),
                                    download_url,
                                )
                        else:
                            console.print(
                                row["purl"],
                            )
        # handle flaw
        if "cve_id" in output["results"][0]:
            for row in output["results"]:
                console.print(
                    row["title"],
                    row["state"],
                    row["impact"],
                    row["resolution"],
                    no_wrap=True,
                )

        # handle trackers
        if "external_system_id" in output["results"][0]:
            for row in output["results"]:
                console.print(
                    row["external_system_id"],
                    row["type"],
                    row["status"],
                    no_wrap=True,
                )

        # handle products
        if "ofuri" in output["results"][0]:
            for row in output["results"]:
                console.print(
                    Text(row["name"], style="magenta bold u"),
                    row["ofuri"],
                    no_wrap=True,
                )
        # handle channels
        if "relative_url" in output["results"][0]:
            for row in output["results"]:
                console.print(
                    Text(row["name"], style="magenta bold u"),
                    row["type"],
                    row["description"],
                    no_wrap=True,
                )
        # handle builds
        if "build_id" in output["results"][0]:
            for row in output["results"]:
                console.print(
                    Text(str(row["build_id"]), style="magenta bold u"),
                    row["build_type"],
                    Text(row["name"], style="white"),
                    row["created_at"],
                    Text(row["link"], style="i"),
                    no_wrap=False,
                )

    ctx.exit()


def text_output_purls(ctx, output, format):
    if "results" in output and output["count"] > 0:
        # handle component
        if "purl" in output["results"][0]:
            ordered_components = sorted(output["results"], key=lambda d: d["purl"])
            for row in ordered_components:
                if "purl" in row:
                    purl = PackageURL.from_string(row["purl"])
                    if not purl.namespace:
                        component_ns = Text("UPSTREAM", style="bold magenta")
                    else:
                        component_ns = Text(purl.namespace.upper(), style="bold red")
                    if ctx.obj["VERBOSE"] == 0:
                        console.print(
                            component_ns,
                            purl.type.upper(),
                            Text(purl.name, style="bold white"),
                            purl.version,
                            purl.qualifiers.get("arch"),
                            no_wrap=False,
                        )
                    if ctx.obj["VERBOSE"] > 0:
                        console.print(
                            component_ns,
                            purl.type.upper(),
                            Text(purl.name, style="bold white"),
                            purl.version,
                            purl.qualifiers.get("arch"),
                            row["link"],
                            no_wrap=False,
                        )
        ctx.exit()


def text_output_generic(ctx, output, format):
    for k, v in output.items():
        key_name = Text(k)
        key_name.stylize("bold magenta")
        console.print(key_name, " : ", v, no_wrap=True)


def cprint(
    data,
    dest=DEST.CONSOLE,
    filename=None,
    ctx=None,
    show_count: bool = True,
):
    """handle format and output"""
    from griffon import get_config_option

    exclude_products = []
    if get_config_option(ctx.obj["PROFILE"], "exclude"):
        exclude_products = get_config_option(ctx.obj["PROFILE"], "exclude").split("\n")
    logger.debug(f"exclude products = {exclude_products}")

    exclude_components = []
    if get_config_option(ctx.obj["PROFILE"], "exclude_components"):
        exclude_components = get_config_option(ctx.obj["PROFILE"], "exclude_components").split("\n")
    logger.debug(f"exclude_components = {exclude_components}")

    output = raw_json_transform(data, show_count)
    if ctx and ctx.obj["NO_COLOR"]:
        console.no_color = True
    format = OUTPUT_FORMAT.JSON
    if ctx and "FORMAT" in ctx.obj:
        format = OUTPUT_FORMAT(ctx.obj["FORMAT"])
    if format is OUTPUT_FORMAT.TEXT:
        if ctx.info_name == "product-summary":
            text_output_product_summary(ctx, output, format, exclude_products)
        if ctx.info_name == "products-contain-component":
            text_output_products_contain_component(
                ctx, output, format, exclude_products, exclude_components
            )
        if ctx.info_name == "components-contain-component":
            text_output_components_contain_component(ctx, output, format, exclude_components)
        if ctx.info_name == "components-affected-by-flaw":
            text_output_components_affected_by_cve(ctx, output, format)
        if ctx.info_name == "products-affected-by-flaw":
            text_output_products_affected_by_cve(ctx, output, format, exclude_products)
        if ctx.info_name == "get-manifest":
            text_output_get_manifest(ctx, output, format)
        if ctx.info_name == "list":
            text_output_list(ctx, output, format, exclude_components)
        if ctx.info_name == "component-flaws":
            text_output_component_flaws(ctx, output, format)
        if ctx.info_name == "product-flaws":
            text_output_product_flaws(ctx, output, format)
        if ctx.info_name == "provides":
            text_output_purls(ctx, output, format)
        if ctx.info_name == "sources":
            text_output_purls(ctx, output, format)
        if ctx.info_name == "tree":
            text_output_tree(ctx, output)

        # last chance text formatted output
        text_output_generic(ctx, output, format)

    if format is OUTPUT_FORMAT.JSON:
        if dest is DEST.CONSOLE:
            console.print_json(json.dumps(output))

    # if we instructed to open browser, open that up now
    if ctx:
        if "link" in data and "open_browser" in ctx.obj:
            if ctx.obj["open_browser"]:
                click.launch(data["link"])

    exit(0)



"""
osidb entity operations

"""
import json
import logging

import click
from osidb_bindings.bindings.python_client.api.osidb import (
    osidb_api_v1_affects_create,
    osidb_api_v1_affects_list,
    osidb_api_v1_affects_retrieve,
    osidb_api_v1_affects_update,
    osidb_api_v1_flaws_create,
    osidb_api_v1_flaws_list,
    osidb_api_v1_flaws_retrieve,
    osidb_api_v1_flaws_update,
    osidb_api_v1_trackers_list,
    osidb_api_v1_trackers_retrieve,
)
from osidb_bindings.bindings.python_client.models import Affect, Flaw, Tracker
from requests import HTTPError

from griffon import OSIDB_API_URL, OSIDBService, progress_bar
from griffon.commands.entities.helpers import (
    abort_if_false,
    filter_request_fields,
    get_editor,
    multivalue_params_to_csv,
    query_params_options,
    request_body_options,
)
from griffon.output import console, cprint

logger = logging.getLogger("griffon")

default_conditions: dict = {}


@click.group(name="osidb")
@click.pass_context
def osidb_grp(ctx):
    pass


# flaws
@osidb_grp.group(help=f"{OSIDB_API_URL}/osidb/api/v1/flaws")
@click.pass_context
def flaws(ctx):
    """OSIDB Flaws."""


@flaws.command(name="list")
@query_params_options(
    entity="Flaw",
    endpoint_module=osidb_api_v1_flaws_list,
    options_overrides={
        "include_fields": {"type": click.Choice(OSIDBService.get_fields(Flaw))},
        "exclude_fields": {"type": click.Choice(OSIDBService.get_fields(Flaw))},
        "include_meta_attr": {"type": click.Choice(OSIDBService.get_meta_attr_fields(Flaw))},
    },
)
@click.pass_context
@progress_bar
def list_flaws(ctx, **params):
    # TODO: handle pagination
    # TODO: handle output
    session = OSIDBService.create_session()

    params = multivalue_params_to_csv(params)
    data = session.flaws.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@flaws.command(name="get")
@click.option(
    "--id",
    "flaw_id",
    help="Flaw CVE-ID or UUID.",
    required=True,
)
@query_params_options(
    entity="Flaw",
    endpoint_module=osidb_api_v1_flaws_retrieve,
    options_overrides={
        "include_fields": {"type": click.Choice(OSIDBService.get_fields(Flaw))},
        "exclude_fields": {"type": click.Choice(OSIDBService.get_fields(Flaw))},
        "include_meta_attr": {"type": click.Choice(OSIDBService.get_meta_attr_fields(Flaw))},
    },
)
@click.pass_context
@progress_bar
def get_flaw(ctx, flaw_id, **params):
    params = multivalue_params_to_csv(params)

    session = OSIDBService.create_session()
    data = session.flaws.retrieve(flaw_id, **params)
    return cprint(data, ctx=ctx)


@flaws.command(name="update")
@click.option(
    "--id",
    "flaw_id",
    help="Flaw CVE-ID or UUID.",
    required=True,
)
@request_body_options(
    endpoint_module=osidb_api_v1_flaws_update, exclude=["uuid", "trackers", "created_dt"]
)
@click.pass_context
def update_flaw(ctx, flaw_id, **params):
    request_body_type = getattr(osidb_api_v1_flaws_update, "REQUEST_BODY_TYPE", None)
    if request_body_type is None:
        raise click.ClickException(
            "No request body template for Flaw update. "
            "Is correct version of osidb-bindings installed?"
        )

    fields = filter_request_fields(
        request_body_type.get_fields(), exclude=["uuid", "trackers", "created_dt"]
    )
    params = multivalue_params_to_csv(params)

    session = OSIDBService.create_session()

    try:
        data = session.flaws.retrieve(id=flaw_id, include_fields=",".join(fields))
    except Exception as e:
        if ctx.obj["VERBOSE"]:
            console.log(e, e.response.json())
        raise click.ClickException(
            f"Failed to fetch Flaw with ID '{flaw_id}'. "
            "Flaw either does not exist or you have insufficient permissions. "
            "Consider running griffon with -v option for verbose error log."
        )

    data = data.to_dict()
    # remove status data from OSIDB server
    [data.pop(key) for key in ["dt", "env", "revision", "version"]]
    data.update((field, value) for field, value in params.items() if value is not None)

    if ctx.obj["EDITOR"]:
        data = click.edit(text=json.dumps(data, indent=4), editor=get_editor(), require_save=False)
        data = json.loads(data)

    try:
        data = session.flaws.update(id=flaw_id, form_data=data)
    except HTTPError as e:
        if ctx.obj["VERBOSE"]:
            console.log(e, e.response.json())
        raise click.ClickException(
            f"Failed to update Flaw with ID '{flaw_id}'. "
            "You might have insufficient permission or you've supplied malformed data. "
            "Consider running griffon with -v option for verbose error log."
        )
    return cprint(data, ctx=ctx)


@flaws.command(name="create")
@request_body_options(
    endpoint_module=osidb_api_v1_flaws_create,
    exclude=["uuid", "trackers", "created_dt", "updated_dt"],
)
@click.pass_context
def create_flaw(ctx, **params):
    request_body_type = getattr(osidb_api_v1_flaws_create, "REQUEST_BODY_TYPE", None)
    if request_body_type is None:
        raise click.ClickException(
            "No request body template for Flaw create. "
            "Is correct version of osidb-bindings installed?"
        )

    fields = filter_request_fields(
        request_body_type.get_fields(),
        exclude=["uuid", "trackers", "created_dt", "updated_dt"],
    )
    params = multivalue_params_to_csv(params)

    session = OSIDBService.create_session()

    data = {field: "" for field in fields}
    data.update((field, value) for field, value in params.items() if value is not None)

    if ctx.obj["EDITOR"]:
        data = click.edit(
            text=json.dumps(data, indent=4, default=str), editor=get_editor(), require_save=False
        )
        data = json.loads(data)

    try:
        data = session.flaws.create(form_data=data)
    except HTTPError as e:
        if ctx.obj["VERBOSE"]:
            console.log(e, e.response.json())
        raise click.ClickException(
            "Failed to create Flaw. "
            "You might have insufficient permission or you've supplied malformed data. "
            "Consider running griffon with -v option for verbose error log."
        )
    return cprint(data, ctx=ctx)


# affects
@osidb_grp.group(help=f"{OSIDB_API_URL}/osidb/api/v1/affects")
@click.pass_context
def affects(ctx):
    """OSIDB Affects."""
    pass


@affects.command(name="list")
@query_params_options(
    entity="Affect",
    endpoint_module=osidb_api_v1_affects_list,
    options_overrides={
        "include_fields": {"type": click.Choice(OSIDBService.get_fields(Affect))},
        "exclude_fields": {"type": click.Choice(OSIDBService.get_fields(Affect))},
        "include_meta_attr": {"type": click.Choice(OSIDBService.get_meta_attr_fields(Affect))},
    },
)
@click.pass_context
@progress_bar
def list_affects(ctx, **params):
    # TODO: handle pagination
    # TODO: handle output
    session = OSIDBService.create_session()

    params = multivalue_params_to_csv(params)
    data = session.affects.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@affects.command(name="get")
@click.option("--uuid", "affect_uuid", help="Affect UUID.", required=True)
@query_params_options(
    entity="Affect",
    endpoint_module=osidb_api_v1_affects_retrieve,
    options_overrides={
        "include_fields": {"type": click.Choice(OSIDBService.get_fields(Affect))},
        "exclude_fields": {"type": click.Choice(OSIDBService.get_fields(Affect))},
        "include_meta_attr": {"type": click.Choice(OSIDBService.get_meta_attr_fields(Affect))},
    },
)
@click.pass_context
@progress_bar
def get_affect(ctx, affect_uuid, **params):
    """
    For parameter reference see:
    <OSIDB_API_URL>/osidb/api/v1/schema/swagger-ui - /osidb/api/v1/affects/{uuid}
    """
    params = multivalue_params_to_csv(params)

    session = OSIDBService.create_session()
    data = session.affects.retrieve(affect_uuid, **params)
    return cprint(data, ctx=ctx)


@affects.command(name="update")
@click.option("--uuid", "affect_uuid", help="Affect UUID.", required=True)
@request_body_options(endpoint_module=osidb_api_v1_affects_update, exclude=["uuid"])
@click.pass_context
def update_affect(ctx, affect_uuid, **params):
    request_body_type = getattr(osidb_api_v1_affects_update, "REQUEST_BODY_TYPE", None)
    if request_body_type is None:
        raise click.ClickException(
            "No request body template for Affect update. "
            "Is correct version of osidb-bindings installed?"
        )

    fields = filter_request_fields(request_body_type.get_fields(), exclude=["uuid"])
    params = multivalue_params_to_csv(params)

    session = OSIDBService.create_session()

    try:
        data = session.affects.retrieve(id=affect_uuid, include_fields=",".join(fields))
    except Exception as e:
        if ctx.obj["VERBOSE"]:
            console.log(e, e.response.json())
        raise click.ClickException(
            f"Failed to fetch Affect with ID '{affect_uuid}'. "
            "Affect either does not exist or you have insufficient permissions. "
            "Consider running griffon with -v option for verbose error log."
        )

    data = data.to_dict()
    # remove status data from OSIDB server
    [data.pop(key) for key in ["dt", "env", "revision", "version"]]
    data.update((field, value) for field, value in params.items() if value is not None)

    if ctx.obj["EDITOR"]:
        data = click.edit(text=json.dumps(data, indent=4), editor=get_editor(), require_save=False)
        data = json.loads(data)

    try:
        data = session.affects.update(id=affect_uuid, form_data=data)
    except HTTPError as e:
        if ctx.obj["VERBOSE"]:
            console.log(e, e.response.json())
        raise click.ClickException(
            f"Failed to update Affect with ID '{affect_uuid}'. "
            "You might have insufficient permission or you've supplied malformed data. "
            "Consider running griffon with -v option for verbose error log."
        )
    return cprint(data, ctx=ctx)


@affects.command(name="create")
@request_body_options(
    endpoint_module=osidb_api_v1_affects_create,
    exclude=["uuid", "created_dt", "updated_dt"],
)
@click.pass_context
def create_affect(ctx, **params):
    request_body_type = getattr(osidb_api_v1_affects_create, "REQUEST_BODY_TYPE", None)
    if request_body_type is None:
        raise click.ClickException(
            "No request body template for Affect create. "
            "Is correct version of osidb-bindings installed?"
        )

    fields = filter_request_fields(
        request_body_type.get_fields(),
        exclude=["uuid", "created_dt", "updated_dt"],
    )
    params = multivalue_params_to_csv(params)

    session = OSIDBService.create_session()

    data = {field: "" for field in fields}
    data.update((field, value) for field, value in params.items() if value is not None)

    if ctx.obj["EDITOR"]:
        data = click.edit(
            text=json.dumps(data, indent=4, default=str), editor=get_editor(), require_save=False
        )
        data = json.loads(data)

    try:
        data = session.affects.create(form_data=data)
    except HTTPError as e:
        if ctx.obj["VERBOSE"]:
            console.log(e, e.response.json())
        raise click.ClickException(
            "Failed to create Affect. "
            "You might have insufficient permission or you've supplied malformed data. "
            "Consider running griffon with -v option for verbose error log."
        )
    return cprint(data, ctx=ctx)


@affects.command(name="delete")
@click.option("--uuid", "affect_uuid", help="Affect UUID.", required=True)
@click.option(
    "--yes",
    is_flag=True,
    callback=abort_if_false,
    expose_value=False,
    prompt="Are you sure you delete affect?",
)
@click.pass_context
def delete_affect(ctx, affect_uuid, **params):
    session = OSIDBService.create_session()
    try:
        data = session.affects.delete(affect_uuid)
    except HTTPError as e:
        if ctx.obj["VERBOSE"]:
            console.log(e, e.response.json())
        raise click.ClickException(
            f"Failed to delete {affect_uuid}. "
            "It either does not exist or you have insufficient permissions."
        )
    return cprint(data, ctx=ctx)


# trackers
@osidb_grp.group(help=f"{OSIDB_API_URL}/osidb/api/v1/trackers")
@click.pass_context
def trackers(ctx):
    """OSIDB Trackers."""
    pass


@trackers.command(name="list")
@query_params_options(
    entity="Tracker",
    endpoint_module=osidb_api_v1_trackers_list,
    options_overrides={
        "include_fields": {"type": click.Choice(OSIDBService.get_fields(Tracker))},
        "exclude_fields": {"type": click.Choice(OSIDBService.get_fields(Tracker))},
        "include_meta_attr": {"type": click.Choice(OSIDBService.get_meta_attr_fields(Tracker))},
    },
)
@click.pass_context
def list_trackers(ctx, **params):
    # TODO: handle pagination
    # TODO: handle output
    session = OSIDBService.create_session()

    params = multivalue_params_to_csv(params)
    data = session.trackers.retrieve_list(**params).results
    return cprint(data, ctx=ctx)


@trackers.command(name="get")
@click.option("--uuid", "tracker_uuid", help="Tracker UUID.", required=True)
@query_params_options(
    entity="Tracker",
    endpoint_module=osidb_api_v1_trackers_retrieve,
    options_overrides={
        "include_fields": {"type": click.Choice(OSIDBService.get_fields(Tracker))},
        "exclude_fields": {"type": click.Choice(OSIDBService.get_fields(Tracker))},
        "include_meta_attr": {"type": click.Choice(OSIDBService.get_meta_attr_fields(Tracker))},
    },
)
@click.pass_context
@progress_bar
def get_tracker(ctx, tracker_uuid, **params):
    params = multivalue_params_to_csv(params)

    session = OSIDBService.create_session()
    data = session.trackers.retrieve(tracker_uuid, **params)
    return cprint(data, ctx=ctx)


@osidb_grp.group(name="admin")
@click.pass_context
def manage_grp(ctx):
    """Manage osidb"""
    pass


@manage_grp.command(name="status")
@click.pass_context
def osidb_status(ctx):
    session = OSIDBService.create_session()
    data = session.status()
    return cprint(data, ctx=ctx)


@manage_grp.command(name="api_doc")
def osidb_api_docs():
    click.launch(f"{OSIDB_API_URL}/osidb/api/v1/schema/swagger-ui/")



"""

"""
import logging
import os

import click

from griffon import get_config_option

logger = logging.getLogger("griffon")

plugin_folder = os.path.join(os.path.dirname(__file__), "plugins")

custom_plugin_dir = get_config_option("default", "custom_plugin_dir", "~/.griffon/plugins/")
custom_folder = os.path.dirname(os.path.expanduser(custom_plugin_dir))


class plugin_commands(click.MultiCommand):
    def list_commands(self, ctx):
        """Dynamically generate list of commands."""
        rv = []
        for filename in os.listdir(plugin_folder):
            if filename.endswith(".py") and not filename.startswith("__init__"):
                rv.append(filename[:-3])
        try:
            for filename in os.listdir(custom_folder):
                if filename.endswith(".py") and not filename.startswith("__init__"):
                    rv.append(filename[:-3])
        except FileNotFoundError:
            click.echo("")
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        """Invoke command."""
        ns = {}
        try:
            fn = os.path.join(plugin_folder, name + ".py")
            with open(fn) as f:
                code = compile(f.read(), fn, "exec")
                eval(code, ns, ns)
            return ns["plugins"]
        except FileNotFoundError:
            try:
                fn = os.path.join(custom_folder, name + ".py")
                with open(fn) as f:
                    code = compile(f.read(), fn, "exec")
                    eval(code, ns, ns)
                return ns["plugins"]
            except FileNotFoundError:
                logger.warning("plugin does not exist.")



import pytest
from click.testing import CliRunner

from griffon.cli import cli

pytestmark = pytest.mark.unit

# @pytest.skip
# def test_cli_verbose():
#     runner = CliRunner()
#     result = runner.invoke(cli, ["-v"])
#     assert result.exit_code == 0, result.output
#
#
# @pytest.skip
# def test_cli_version():
#     runner = CliRunner()
#     result = runner.invoke(cli, ["-V"])
#     assert result.exit_code == 0
#     assert result.output == "griffon 0.1.0\n"
#


def test_cli_components():
    runner = CliRunner()
    result = runner.invoke(cli, ["--help"])
    assert result.exit_code == 0



"""

"""
import logging

import click
import click_completion

from griffon import (
    config_logging,
    get_config_option,
    list_config_sections,
    print_version,
)

from .commands.configure import configure_grp
from .commands.docs import docs_grp
from .commands.entities import entities_grp
from .commands.plugin_commands import plugin_commands
from .commands.queries import queries_grp
from .output import OUTPUT_FORMAT

logger = logging.getLogger("griffon")

click_completion.init()


@click.group()
@click.pass_context
def configure(ctx):
    pass


configure.add_command(configure_grp)


@click.group()
@click.pass_context
def entities(ctx):
    pass


entities.add_command(entities_grp)


@click.group()
@click.pass_context
def services_grp(ctx):
    pass


services_grp.add_command(queries_grp)


@click.group()
@click.pass_context
def docs(ctx):
    pass


docs.add_command(docs_grp)


@click.group()
@click.pass_context
def plugins(ctx):
    pass


@plugins.command(name="plugins", help="3rd party plugins.", cls=plugin_commands)
@click.pass_context
def plugins_grp(ctx):
    pass


# CLI entry point
#
#   A click.CommandCollection is used to aggregate up all CLI sub commands.
#   Top level CLI options (germane to all commands) are included here.


@click.group(
    cls=click.CommandCollection,
    sources=(configure, entities, services_grp, docs, plugins),
)
@click.option(
    "--version",
    "-V",
    is_flag=True,
    callback=print_version,
    expose_value=False,
    is_eager=True,
    help="Display griffon version.",
)
@click.option("--debug", "-d", is_flag=True, help="Debug log level.")
@click.option(
    "--format",
    "-f",
    type=click.Choice([el.value for el in OUTPUT_FORMAT]),
    default=get_config_option("default", "format", "text"),
    help="Result format (default is text format).",
)
@click.option(
    "-v",
    "verbose",
    count=True,
    default=get_config_option("default", "verbosity", 0),
    help="Verbose output, more detailed search results, can be used multiple times (e.g. -vvv).",
)  # noqa
@click.option("--no-progress-bar", is_flag=True, help="Disable progress bar.")
@click.option("--no-color", is_flag=True, help="Disable output of color ansi esc sequences.")
@click.option(
    "--profile",
    "profile",
    type=click.Choice(list_config_sections()),
    default=get_config_option(
        "default",
        "profile",
    ),
    help="Activate profile, defined in .griffonrc.",
)
@click.option("--editor/--no-editor", default=True, help="Allow text editor prompt.")
@click.pass_context
def cli(ctx, debug, format, verbose, no_progress_bar, no_color, profile, editor):
    """Red Hat product security CLI"""

    if ctx.invoked_subcommand is None:
        click.echo(ctx.parent.get_help())

    if not debug:
        config_logging(level="INFO")
    else:
        config_logging(level="DEBUG")

    ctx.ensure_object(dict)
    ctx.obj["DEBUG"] = debug
    ctx.obj["SHOW_INACTIVE"] = False
    ctx.obj["SHOW_PURL"] = False
    ctx.obj["SHOW_UPSTREAM"] = False
    ctx.obj["FORMAT"] = format
    ctx.obj["VERBOSE"] = verbose
    ctx.obj["NO_PROGRESS_BAR"] = no_progress_bar
    ctx.obj["NO_COLOR"] = no_color
    ctx.obj["PROFILE"] = profile
    ctx.obj["SHORT_VERSION_VALUES"] = True
    ctx.obj["EDITOR"] = editor


cli.help = "Red Hat Product Security CLI"



import pytest
from click.testing import CliRunner

from griffon.cli import cli

pytestmark = pytest.mark.unit


def test_osidb():
    runner = CliRunner()
    result = runner.invoke(cli, ["entities", "osidb", "admin"])
    assert result.exit_code == 0


def test_corgi():
    runner = CliRunner()
    result = runner.invoke(cli, ["entities", "component-registry", "admin"])
    assert result.exit_code == 0


def test_community_components():
    runner = CliRunner()
    result = runner.invoke(cli, ["entities", "community-component-registry", "admin"])
    assert result.exit_code == 0



"""

"""
import copy
import logging

import click
from component_registry_bindings.bindings.python_client.api.v1 import v1_components_list
from component_registry_bindings.bindings.python_client.models import Component

from griffon import CorgiService, OSIDBService, get_config_option, progress_bar
from griffon.autocomplete import (
    get_component_names,
    get_cve_ids,
    get_product_stream_names,
    get_product_stream_ofuris,
    get_product_version_names,
)
from griffon.commands.entities.corgi import (
    get_component_manifest,
    get_component_summary,
    get_product_stream_manifest,
    list_components,
)
from griffon.commands.entities.helpers import query_params_options
from griffon.commands.reports import (
    generate_affects_report,
    generate_entity_report,
    generate_license_report,
)
from griffon.output import console, cprint, raw_json_transform
from griffon.services import QueryService, core_queries  # , exp

logger = logging.getLogger("griffon")

query_service = QueryService()


@click.group(name="service", help="Service operations.")
@click.pass_context
def queries_grp(ctx):
    """Query operations."""
    pass


queries_grp.add_command(generate_affects_report)
queries_grp.add_command(generate_entity_report)
queries_grp.add_command(generate_license_report)

# queries_grp.add_command(generate_affects_for_component_process)


@queries_grp.command(
    name="product-summary",
    help="Get Product summaries.",
)
@click.argument(
    "product_stream_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
# TODO - underlying bindings need to support 'ofuri'
# @click.option(
#     "--ofuri",
#     "ofuri",
#     type=click.STRING,
#     shell_complete=get_product_stream_ofuris,
#     help="UNDER DEVELOPMENT",
# )
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of name.",
)
@click.option(
    "--all",
    "all",
    is_flag=True,
    default=False,
    help="Return all Products.",
)
@click.option(
    "-v",
    "verbose",
    count=True,
    default=get_config_option("default", "verbosity", 0),
    help="Verbose output, more detailed search results, can be used multiple times (e.g. -vvv).",
)  # noqa
@click.pass_context
@progress_bar
def get_product_summary(ctx, product_stream_name, strict_name_search, all, verbose):
    """get product stream."""
    if verbose:
        ctx.obj["VERBOSE"] = verbose
    if not product_stream_name and not all and not strict_name_search:
        click.echo(ctx.get_help())
        exit(0)
    q = query_service.invoke(core_queries.product_stream_summary, ctx.params)
    cprint(q, ctx=ctx)


@queries_grp.command(
    name="component-summary",
    help="Get Component summaries.",
)
@click.argument(
    "component_name",
    required=False,
    type=click.STRING,
    shell_complete=get_component_names,
)
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of name.",
)
@click.pass_context
def retrieve_component_summary(ctx, component_name, strict_name_search):
    """Get Component summary."""
    if not component_name:
        click.echo(ctx.get_help())
        exit(0)
    cond = {}
    if component_name:
        cond["component_name"] = component_name
    ctx.invoke(get_component_summary, **cond)


@queries_grp.command(
    name="products-contain-component",
    help="List Products containing Component.",
)
@click.argument(
    "component_name",
    required=False,
)
@click.option(
    "--purl", help="Component purl, needs to be in quotes (ex. 'pkg:rpm/python-pyjwt@1.7.1')"
)
@click.option(
    "--arch",
    default="src",
    type=click.Choice(CorgiService.get_component_arches()),
)
@click.option(
    "--namespace", default=None, type=click.Choice(CorgiService.get_component_namespaces())
)
@click.option("--type", "component_type", type=click.Choice(CorgiService.get_component_types()))
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.option(
    "--generate-affects",
    "-a",
    "affect_mode",
    is_flag=True,
    default=False,
    help="Generate Affects output.",
)
# @click.option(
#     "--cve-id",
#     "cve_id",
#     help="Attach affects to this (\033[1mCVE-ID\033[0m).",
# )
@click.option(
    "--sfm2-flaw-id",
    "sfm2_flaw_id",
    help="Attach affects to this (\033[1msfm2 flaw id\033[0m).",
)
@click.option(
    "--flaw-mode",
    "flaw_mode",
    default="dry_run",
    type=click.Choice(["add", "replace", "dry_run"]),
    help="Add or update when generating affects.",
)
@click.option(
    "--search-latest",
    "search_latest",
    is_flag=True,
    default=False,
    help="Search root Components (\033[1menabled by default\033[0m).",
)
@click.option(
    "--search-related-url",
    "search_related_url",
    is_flag=True,
    default=False,
    help="Search related url (\033[1menabled by default\033[0m).",
)
@click.option(
    "--filter-rh-naming",
    is_flag=True,
    default=False,
    help="Filter rh naming.",
)
@click.option(
    "--search-all",
    "search_all",
    is_flag=True,
    default=False,
    help="Search all root Components and dependencies.",
)
@click.option(
    "--search-redhat",
    "search_redhat",
    is_flag=True,
    default=False,
    help="Search all Red Hat root Components and dependencies.",
)
@click.option(
    "--search-community",
    "search_community",
    is_flag=True,
    default=False,
    help="Search community Components.",
)
@click.option(
    "--search-upstreams",
    "search_upstreams",
    is_flag=True,
    default=False,
    help="Search for Components by upstream.",
)
@click.option(
    "-v",
    "verbose",
    count=True,
    default=get_config_option("default", "verbosity", 0),
    help="Verbose output, more detailed search results, can be used multiple times (e.g. -vvv).",
)  # noqa
@click.pass_context
def get_product_contain_component(
    ctx,
    component_name,
    purl,
    arch,
    namespace,
    component_type,
    strict_name_search,
    affect_mode,
    sfm2_flaw_id,
    flaw_mode,
    search_latest,
    search_related_url,
    filter_rh_naming,
    search_all,
    search_redhat,
    search_community,
    search_upstreams,
    verbose,
):
    with console.status("griffoning", spinner="line") as operation_status:
        """List products of a latest component."""
        if verbose:
            ctx.obj["VERBOSE"] = verbose
        if not purl and not component_name:
            click.echo(ctx.get_help())
            click.echo("")
            click.echo("\033[1mMust supply Component name or --purl.\033[0m")
            exit(0)

        if (
            not search_latest
            and not search_all
            and not search_related_url
            and not search_community
            and not search_upstreams
            and not search_redhat
        ):
            ctx.params["search_latest"] = True

        params = copy.deepcopy(ctx.params)
        params.pop("verbose")
        params.pop("sfm2_flaw_id")
        params.pop("flaw_mode")
        params.pop("affect_mode")
        if component_name:
            q = query_service.invoke(core_queries.products_containing_component_query, params)
        if purl:
            q = query_service.invoke(
                core_queries.products_containing_specific_component_query, params
            )

        # TODO - in the short term affect handling will be mediated via sfm2 here in the operation itself # noqa
        if ctx.params["sfm2_flaw_id"]:
            console.no_color = True
            console.highlighter = None
            operation_status.stop()

            # generate affects
            output = raw_json_transform(q, True)
            ordered_results = sorted(output["results"], key=lambda d: d["product_stream"])
            affects = []
            product_versions = sorted(
                list(set([item["product_version"] for item in ordered_results]))
            )
            for pv in product_versions:
                names = [item["name"] for item in ordered_results if pv == item["product_version"]]
                names = list(set(names))
                for name in names:
                    affects.append(
                        {"product_version": pv, "component_name": name, "operation": "add"}
                    )

            # attempt to import sfm2client module
            try:
                import sfm2client
            except ImportError:
                logger.warning("sfm2client library not found, cannot compare with flaw affects")
                ctx.exit()

            # TODO: paramaterise into dotfile/env var
            sfm2 = sfm2client.api.core.SFMApi({"url": "http://localhost:5600"})
            try:
                flaw = sfm2.flaw.get(sfm2_flaw_id)
            except Exception as e:
                logger.warning(f"Could not retrieve flaw {sfm2_flaw_id}: {e}")
                return

            if ctx.params["flaw_mode"] == "replace":
                if affects:
                    console.print(
                        f"The following affects will REPLACE all flaw {flaw['id']}'s existing affects in \"new\" state:\n"  # noqa
                    )
                    for m in affects:
                        console.print(
                            f"{m['product_version']}\t{m['component_name']}",
                            no_wrap=False,
                        )

                    if click.confirm(
                        f"\nREPLACE flaw {flaw['id']}'s existing affects in \"new\" state with the above? THIS CANNOT BE UNDONE: ",  # noqa
                        default=True,
                    ):
                        click.echo("Updating ...")
                        # only discard affects in 'new' state, we should preserve all others so not to throw work away # noqa
                        new_affects = [a for a in flaw["affects"] if a["affected"] != "new"]
                        # get map of existing affects first, so that we don't try to add duplicates
                        existing = set((a["ps_module"], a["ps_component"]) for a in new_affects)
                        for m in affects:
                            if (m["product_version"], m["component_name"]) in existing:
                                continue
                            new_affects.append(
                                {
                                    "affected": "new",
                                    "ps_component": m["component_name"],
                                    "ps_module": m["product_version"],
                                }
                            )
                        try:
                            sfm2.flaw.update(flaw["id"], data={"affects": new_affects})
                        except Exception as e:
                            msg = e.response.json()
                            logger.warning(f"Failed to update flaw: {e}: {msg}")
                        console.print("Operation done.")
                        ctx.exit()

                    click.echo("No affects were added to flaw.")
                else:
                    console.print("No affects to add to flaw.")
            else:
                missing = []
                for affect in affects:
                    flaw_has_affect = False
                    for a in flaw.get("affects"):
                        if a.get("ps_module") == affect.get("product_version") and a.get(
                            "ps_component"
                        ) == affect.get("component_name"):
                            flaw_has_affect = True
                    if not flaw_has_affect:
                        missing.append(affect)

                if missing:
                    console.log("Flaw is missing the following affect entries:\n")
                    for m in missing:
                        console.print(
                            f"{m['product_version']}\t{m['component_name']}",
                            no_wrap=False,
                        )
                    if click.confirm(
                        "Would you like to add them? ",
                        default=True,
                    ):
                        click.echo("Updating ...")

                        updated_affects = flaw.get("affects")[:]
                        for m in missing:
                            updated_affects.append(
                                {
                                    "affected": "new",
                                    "ps_component": m["component_name"],
                                    "ps_module": m["product_version"],
                                }
                            )
                        try:
                            sfm2.flaw.update(flaw["id"], data={"affects": updated_affects})
                        except Exception as e:
                            msg = e.response.json()
                            logger.warning(f"Failed to update flaw: {e}: {msg}")
                        console.print("Operation done.")
                        ctx.exit()
                    click.echo("No affects were added to flaw.")

                else:
                    console.print("Flaw is not missing any affect entries")

            ctx.exit()

        cprint(q, ctx=ctx)


@queries_grp.command(
    name="components-contain-component",
    help="List Components containing Component.",
)
@click.argument("component_name", required=False)
@click.option("--purl")
@click.option("--type", "component_type", type=click.Choice(CorgiService.get_component_types()))
@click.option("--version", "component_version")
@click.option(
    "--arch",
    "component_arch",
    type=click.Choice(CorgiService.get_component_arches()),
    help="Default arch=src.",
)
@click.option("--namespace", type=click.Choice(CorgiService.get_component_namespaces()))
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.option(
    "-v",
    "verbose",
    count=True,
    default=get_config_option("default", "verbosity", 0),
    help="Verbose output, more detailed search results, can be used multiple times (e.g. -vvv).",
)  # noqa
@click.pass_context
@progress_bar
def get_component_contain_component(
    ctx,
    component_name,
    purl,
    component_type,
    component_version,
    component_arch,
    namespace,
    strict_name_search,
    verbose,
):
    """List components that contain component."""
    if verbose:
        ctx.obj["VERBOSE"] = verbose
    if not component_name and not purl:
        click.echo(ctx.get_help())
        exit(0)
    if component_name:
        q = query_service.invoke(core_queries.components_containing_component_query, ctx.params)
        cprint(q, ctx=ctx)
    if purl:
        q = query_service.invoke(
            core_queries.components_containing_specific_component_query, ctx.params
        )
        cprint(q, ctx=ctx)


@queries_grp.command(
    name="product-manifest",
    help="Get Product manifest (includes Root Components and all dependencies).",
)
@click.argument("product_stream_name", required=False, shell_complete=get_product_stream_names)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
@click.option(
    "--spdx-json",
    "spdx_json_format",
    is_flag=True,
    default=False,
    help="Generate spdx manifest (json).",
)
@click.pass_context
def get_product_manifest_query(ctx, product_stream_name, ofuri, spdx_json_format):
    """List components of a specific product version."""
    if not ofuri and not product_stream_name:
        click.echo(ctx.get_help())
        exit(0)

    if spdx_json_format:
        ctx.ensure_object(dict)
        ctx.obj["FORMAT"] = "json"  # TODO - investigate if we need yaml format.

    cond = {}
    if ofuri:
        cond["ofuri"] = ofuri
    if product_stream_name:
        cond["product_stream_name"] = product_stream_name
    ctx.invoke(get_product_stream_manifest, **cond)


@queries_grp.command(
    name="product-components",
    help="List LATEST Root Components of Product.",
)
@click.argument("product_stream_name", required=False, shell_complete=get_product_stream_names)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
@query_params_options(
    entity="Component",
    endpoint_module=v1_components_list,
    options_overrides={
        "include_fields": {"type": click.Choice(CorgiService.get_fields(Component))},
    },
)
@click.option(
    "-v",
    "verbose",
    count=True,
    default=get_config_option("default", "verbosity", 0),
    help="Verbose output, more detailed search results, can be used multiple times (e.g. -vvv).",
)  # noqa
@click.pass_context
def get_product_latest_components_query(ctx, product_stream_name, ofuri, verbose, **params):
    """List components of a specific product version."""
    if verbose:
        ctx.obj["VERBOSE"] = verbose
        ctx.params.pop("verbose")
    if not ofuri and not product_stream_name:
        click.echo(ctx.get_help())
        exit(0)
    if ofuri:
        params["ofuri"] = ofuri
    if product_stream_name:
        # lookup ofuri
        session = CorgiService.create_session()
        ps = session.product_streams.retrieve_list(name=product_stream_name)
        params["ofuri"] = ps["ofuri"]
    ctx.invoke(list_components, **params)


@queries_grp.command(
    name="component-manifest",
    help="Get Component manifest.",
)
@click.option("--uuid", "component_uuid")
@click.option("--purl", help="Component Purl (must be quoted).")
@click.option(
    "--spdx-json",
    "spdx_json_format",
    is_flag=True,
    default=False,
    help="Generate spdx manifest (json).",
)
@click.pass_context
def retrieve_component_manifest(ctx, component_uuid, purl, spdx_json_format):
    """Retrieve Component manifest."""
    if not component_uuid and not purl:
        click.echo(ctx.get_help())
        exit(0)
    if spdx_json_format:
        ctx.ensure_object(dict)
        ctx.obj["FORMAT"] = "json"
    cond = {}
    if component_uuid:
        cond["uuid"] = component_uuid
    if purl:
        cond["purl"] = purl
    ctx.invoke(get_component_manifest, **cond)


@queries_grp.command(
    name="components-affected-by-flaw",
    help="List Components affected by Flaw.",
)
@click.argument("cve_id", required=False, type=click.STRING, shell_complete=get_cve_ids)
@click.option(
    "--affectedness",
    help="Filter by Affect affectedness.",
    type=click.Choice(OSIDBService.get_affect_affectedness()),
)
@click.option(
    "--resolution",
    "affect_resolution",
    help="Filter by Affect resolution.",
    type=click.Choice(OSIDBService.get_affect_resolution()),
)
@click.option(
    "--impact",
    "affect_impact",
    help="Filter by Affect impact.",
    type=click.Choice(OSIDBService.get_affect_impact()),
)
@click.option(
    "--type",
    "component_type",
    type=click.Choice(CorgiService.get_component_types()),
    help="Filter by Component type.",
)
@click.option(
    "--namespace",
    type=click.Choice(CorgiService.get_component_namespaces()),
    help="filter by Component namespace.",
)
@click.pass_context
@progress_bar
def components_affected_by_specific_cve_query(
    ctx, cve_id, affectedness, affect_resolution, affect_impact, component_type, namespace
):
    """List components affected by specific CVE."""
    if not cve_id:
        click.echo(ctx.get_help())
        exit(0)
    q = query_service.invoke(core_queries.components_affected_by_specific_cve_query, ctx.params)
    cprint(q, ctx=ctx)


@queries_grp.command(
    name="products-affected-by-flaw",
    help="List Products affected by Flaw.",
)
@click.argument("cve_id", required=False, type=click.STRING, shell_complete=get_cve_ids)
@click.pass_context
@progress_bar
def product_versions_affected_by_cve_query(ctx, cve_id):
    """List Products affected by a CVE."""
    if not cve_id:
        click.echo(ctx.get_help())
        exit(0)
    q = query_service.invoke(
        core_queries.products_versions_affected_by_specific_cve_query, ctx.params
    )
    cprint(q, ctx=ctx)


@queries_grp.command(name="component-flaws", help="List Flaws affecting a Component.")
@click.argument("component_name", required=False)
@click.option("--purl")
@click.option(
    "--flaw-state",
    "flaw_state",
    help="Filter by Flaw state.",
    type=click.Choice(OSIDBService.get_flaw_states()),
)
@click.option(
    "--flaw-impact",
    "flaw_impact",
    help="Filter by Flaw impact.",
    type=click.Choice(OSIDBService.get_flaw_impacts()),
)
@click.option(
    "--flaw-resolution",
    "flaw_resolution",
    help="Filter by Flaw resolution.",
    type=click.Choice(OSIDBService.get_flaw_resolutions()),
)
@click.option(
    "--affectedness",
    help="Filter by Affect affectedness.",
    type=click.Choice(OSIDBService.get_affect_affectedness()),
)
@click.option(
    "--affect-resolution",
    "affect_resolution",
    help="Filter by Affect resolution.",
    type=click.Choice(OSIDBService.get_affect_resolution()),
)
@click.option(
    "--affect-impact",
    "affect_impact",
    help="Filter by Affect impact.",
    type=click.Choice(OSIDBService.get_affect_impact()),
)
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.pass_context
@progress_bar
def cves_for_specific_component_query(
    ctx,
    component_name,
    purl,
    flaw_state,
    flaw_impact,
    flaw_resolution,
    affectedness,
    affect_resolution,
    affect_impact,
    strict_name_search,
):
    """List flaws of a specific component."""
    if not purl and not component_name:
        click.echo(ctx.get_help())
        exit(0)

    q = query_service.invoke(core_queries.cves_for_specific_component_query, ctx.params)
    cprint(q, ctx=ctx)


@queries_grp.command(
    name="product-flaws",
    help="List Flaws affecting a Product.",
)
@click.argument(
    "product_version_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_version_names,
)
@click.option("--ofuri")
@click.option(
    "--flaw-state",
    "flaw_state",
    help="Filter by Flaw state.",
    type=click.Choice(OSIDBService.get_flaw_states()),
)
@click.option(
    "--flaw-impact",
    "flaw_impact",
    help="Filter by Flaw impact.",
    type=click.Choice(OSIDBService.get_flaw_impacts()),
)
@click.option(
    "--flaw-resolution",
    "flaw_resolution",
    help="Filter by Flaw resolution.",
    type=click.Choice(OSIDBService.get_flaw_resolutions()),
)
@click.option(
    "--affectedness",
    help="Filter by Affect affectedness.",
    type=click.Choice(OSIDBService.get_affect_affectedness()),
)
@click.option(
    "--affect-resolution",
    "affect_resolution",
    help="Filter by Affect resolution.",
    type=click.Choice(OSIDBService.get_affect_resolution()),
)
@click.option(
    "--affect-impact",
    "affect_impact",
    help="Filter by Affect impact.",
    type=click.Choice(OSIDBService.get_affect_impact()),
)
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.pass_context
@progress_bar
def cves_for_specific_product_query(
    ctx,
    product_version_name,
    ofuri,
    flaw_state,
    flaw_impact,
    flaw_resolution,
    affectedness,
    affect_impact,
    affect_resolution,
    strict_name_search,
):
    """List flaws of a specific product."""
    if not product_version_name and not ofuri:
        click.echo(ctx.get_help())
        exit(0)
    q = query_service.invoke(core_queries.cves_for_specific_product_query, ctx.params)
    cprint(q, ctx=ctx)



import logging

import click

from .community_component_registry import commmunity_components_grp
from .corgi import corgi_grp
from .osidb import osidb_grp

logger = logging.getLogger("griffon")


default_conditions: dict = {}


@click.group(name="entities", help="Entity operations.")
@click.option("--open-browser", is_flag=True, help="open browser to service results.")
@click.option("--limit", default=10, help="# of items returned by list operations.")
@click.pass_context
def entities_grp(ctx, open_browser, limit):
    ctx.ensure_object(dict)
    ctx.obj["open_browser"] = open_browser
    ctx.obj["limit"] = limit


entities_grp.add_command(osidb_grp)
entities_grp.add_command(corgi_grp)
entities_grp.add_command(commmunity_components_grp)
